package com.blogspot.thedsweb.engine;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.Scalar;
import org.opencv.highgui.VideoCapture;

import com.blogspot.thedsweb.util.Config;
import com.blogspot.thedsweb.util.Files;
import com.blogspot.thedsweb.util.Initialization;

public class Brightness extends Files {
	
	private int min;
	private int max;
	private static int last;
	private int current;
	private int maxLevel;
	private double[] referenceValue;
	private int[] brightnessValue;
	private String path;
	private boolean face;
	private static int meanValue;
	private Threshold threshold;
	private Face probability;
	
	static {
		//Load native library of OpenCV
		System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
	}

	public Brightness(Config config) {
		//Create threshold object to calculate threshold
		threshold = new Threshold(config);
		
		//Create Face object to track faces
		probability = new Face();
		
		//Initialize Files if they dosen't exist
		Initialization.Files();
		int[] arr = new int[4];
		path = readStatus(arr);
		min = arr[0];
		max = arr[1];
		last = arr[2];
		maxLevel = arr[3];
		setReference();
		setBrightnessStep();
		current = captureAndCalculate();
	}
	
	//Return the current Brightness
	public int getCurrent() {
		return current;
	}
	
	//Get the current Brightness
	public void setCurrent() {
		current = captureAndCalculate();
	}
	
	//Return the last set Brightness
	public int getLast() {
		return last;
	}
	
	//Return if Faces are detected
	public boolean getFace() {
		return face;
	}
	
	// Control the difference between last and 
	// current Brightness value return true when
	// it is too high
	public boolean control(){
		return current > last<<1 || current<<1 < last;
	}
	
	private void setReference() {
		//Reference mean values
		double referenceStep = (max^min)/maxLevel;
		double[] referenceValue = new double[maxLevel+1];
		referenceValue[0] = max-referenceStep;
		for ( int i = 0; i < referenceValue.length-1; i++ ) {
			if ( referenceValue[i]-referenceStep < 0 ) {
				referenceValue[i+1] = 0;
			} else {
				referenceValue[i+1] = referenceValue[i]-referenceStep;
			}
		}
		this.referenceValue = referenceValue;
	}
	
	private void setBrightnessStep() {
		//Brightness steps
		int maxBrightnessLevel = maxLevel;
		int[] brightnessValue = new int[maxLevel+1];
		for ( int i = 0; i < brightnessValue.length; i++ ) {
			brightnessValue[i] = maxBrightnessLevel--;
		}
		this.brightnessValue = brightnessValue;
	}
	
	private int captureAndCalculate() {	
		Mat frame = new Mat();
		VideoCapture cap = new VideoCapture(0);
		
		
		if ( !cap.isOpened() ) {
			face = true;
			meanValue = current;
		}
		
		// Grab and decode camera frame and release camera for other programs
		try {
			//Time to initialize camera
			Thread.sleep(500);
			
			//Drop first 9 Frames
			for ( int i = 0; i <= 10; i++ ) {
				cap.read(frame);
				Thread.sleep(5);
			}
			
			//Readout average brightness values of frame per Color in RGB space 
			Scalar mean = Core.mean(frame);
			
			//Calculate mean value of frame
			meanValue = meanOfScalar(mean.val[0],mean.val[1],mean.val[2]);
			
			//Set true if someone's Face is detected
			face = probability.detectFace(frame, meanValue);
		} catch ( InterruptedException e ) {
			//if interrupted try to return the last value of current
			face = true;
			meanValue = current;
		} finally {
			cap.release();
		}
		
		/* 
		*  If the frame is completely black or
		*  white the camera must already in use 
		*  by another program 
		*/
		if ( meanValue == 0 ) {
			face = true;
			meanValue = current;
		}

		return meanValue;
	}
	
	private int meanOfScalar(double r, double g, double b) {
		//Standard calculation of brightness in RGB space
		int result = (int)(0.2126*r + 0.7152*g + 0.0722*b);
		return result;
	}
	
	public void setBrightness() {	
		//Threshold values to prevent flickering
		int darkeningThreshold = threshold.getDarkeningThreshold(last);
		int brighteningThreshold = threshold.getBrighteningThreshold(last);
		
		//Set the display brightness value relative to reference
		if ( current <= darkeningThreshold || current > brighteningThreshold ) {
			//Set Brightness
			for ( int i = 0, minCheck = brightnessValue.length-1; i < brightnessValue.length; i++ ) {
				if ( current > referenceValue[i] || i == minCheck) {
					writeInt(brightnessValue[i], path);
					break;
				}
			}
			
			// Check if the new current value is also the new
			// minimum or maximum value
			if ( current < min ) {
				min = current;
				setReference();
			}
			if ( current > max ) {
				max = current;
				setReference();
			}
			
			//Save last current value
			last = current;
			
			//Save all values
			saveValue(current, min, max);
		}
	}
	
	public int calibrateBrightness() {
		//Get new current value
		setCurrent();
		
		// Check if the new current value is also the new
		// minimum or maximum value
		if ( current < min ) {
			min = current;;
		}
		if ( current > max ) {
			max = current;
		}
		
		//Save all values
		saveValue(current, min, max);
		
		return current;
	}
	
	public void darkenBrightness() {
		last = 0;
		writeInt(brightnessValue[maxLevel], path);
	}
}
